package com.sample;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.builder.conf.DeclarativeAgendaOption;
import org.drools.event.rule.AfterActivationFiredEvent;
import org.drools.event.rule.DefaultAgendaEventListener;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;



/**
 * 
 * Class to reproduce declarative agenda cancel error
 * in StatefulKnowledgeSession.
 * 
 * When is activation canceling used in the first iteration,
 * than it can cancel activations of a rule specified by a condition.
 * 
 * But this does not work for the second iteration. The rule is not canceled.
 * The canceling rule is fired as the last, even if it has 'direct'
 * activation listener.
 * 
 * @author jsvitak@redhat.com
 * @version BRMS 5.3.0 DEV4
 *
 */
public class DeclarativeAgendaCancel {
    
    public static void main(String args[]) {
            KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
            kbuilder.add(ResourceFactory.newFileResource(new File("test-resources", "declarative-agenda-cancel.drl")), ResourceType.DRL);
            
            //System.out.println(kbuilder.getErrors().toString());

            KnowledgeBaseConfiguration kconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
            kconf.setOption(DeclarativeAgendaOption.ENABLED);
            KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(kconf);
            kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

            StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
            OrderListener listener = new OrderListener();
            ksession.addEventListener(listener);

           
            // first run - without cancel rule, sales1 and sales2 are fired
            ksession.insert("fireRules");
            ksession.fireAllRules();
            //assertEquals(listener.size(), 2);
            //String[] expected = {"sales1", "sales2"};
            for (int i = 0; i < listener.size(); i++) {
                System.out.println(listener.get(i));
                //assertEquals(listener.get(i), expected[i]);
            }

            // second run - added cancelRule, so it should fire just cancelRule and sales1 rule
            // but it fires sales1, sales2 and salesCancel, so salesCancel did not cancel sales2 activation
            System.out.println("#### second run: ####");
            ksession.insert("fireCancelRule");
            ksession.fireAllRules();
            //assertEquals(listener.size(), 2);
            //String[] expected2 = { "salesCancel", "sales1"};
            for (int i = 0; i < listener.size(); i++) {
                System.out.println(listener.get(i));
                //assertEquals(listener.get(i), expected2[i]);
            }
            
        
    }
    
    
    public static class OrderListener extends DefaultAgendaEventListener {
        private List<String> rulesFired = new ArrayList<String>();

        @Override
        public void afterActivationFired(AfterActivationFiredEvent event) {
            rulesFired.add(event.getActivation().getRule().getName());
        }

        public int size() {
            return rulesFired.size();
        }

        public String get(int index) {
            return rulesFired.get(index);
        }
    }
    
}
